﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProjectTractor.Entities;

namespace ProjectTractor.DrawningObjects
{
    /// <summary>
    /// Класс, отвечающий за прорисовку и перемещение объекта-сущности
    /// </summary>
    public class DrawningBulldoser : DrawningTractor
    {
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес</param>
        /// <param name="bodyColor">Основной цвет</param>
        /// <param name="additionalColor">Дополнительный цвет</param>
        /// <param name="wheelsOrnament">Признак наличия орнамента на колесах</param>
        /// <param name="blade">Признак наличия отвала</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        public DrawningBulldoser(int speed, double weight, Color bodyColor, Color
       additionalColor, bool wheelsOrnament, bool blade, int width, int height) :
        base(speed, weight, bodyColor, width, height, 110, 60)
        {
            if (EntityTractor != null)
            {
                EntityTractor = new EntityBulldoser(speed, weight, bodyColor,
                additionalColor, wheelsOrnament, blade);
            }
        }
        public override void DrawTransport(Graphics g)
        {
            if (EntityTractor is not EntityBulldoser bulldoser)
            {
                return;
            }
            Pen pen = new(Color.Black);
            Brush additionalBrush = new
            SolidBrush(bulldoser.AdditionalColor);
            
            base.DrawTransport(g);
            // орнамент на колесах
            if (bulldoser.WheelsOrnament)
            {
                g.FillEllipse(additionalBrush, _startPosX + 2, _startPosY + 47, 11, 11);
                g.FillEllipse(additionalBrush, _startPosX + 67, _startPosY + 47, 11, 11);
                g.FillEllipse(additionalBrush, _startPosX + 37, _startPosY + 57, 6, 6);
                g.FillEllipse(additionalBrush, _startPosX + 22, _startPosY + 57, 6, 6);
                g.FillEllipse(additionalBrush, _startPosX + 52, _startPosY + 57, 6, 6);
                g.FillEllipse(additionalBrush, _startPosX + 27, _startPosY + 42, 6, 6);
                g.FillEllipse(additionalBrush, _startPosX + 47, _startPosY + 42, 6, 6);
                g.DrawLine(pen, _startPosX + 7, _startPosY + 47, _startPosX + 7, _startPosY + 58);
                g.DrawLine(pen, _startPosX + 2, _startPosY + 53, _startPosX + 13, _startPosY + 53);

                g.DrawLine(pen, _startPosX + 73, _startPosY + 47, _startPosX + 73, _startPosY + 58);
                g.DrawLine(pen, _startPosX + 67, _startPosY + 53, _startPosX + 78, _startPosY + 53);

                g.DrawLine(pen, _startPosX + 40, _startPosY + 57, _startPosX + 40, _startPosY + 62);
                g.DrawLine(pen, _startPosX + 37, _startPosY + 60, _startPosX + 43, _startPosY + 60);

                g.DrawLine(pen, _startPosX + 25, _startPosY + 57, _startPosX + 25, _startPosY + 62);
                g.DrawLine(pen, _startPosX + 22, _startPosY + 60, _startPosX + 28, _startPosY + 60);

                g.DrawLine(pen, _startPosX + 55, _startPosY + 57, _startPosX + 55, _startPosY + 62);
                g.DrawLine(pen, _startPosX + 52, _startPosY + 60, _startPosX + 58, _startPosY + 60);

                g.DrawLine(pen, _startPosX + 30, _startPosY + 42, _startPosX + 30, _startPosY + 48);
                g.DrawLine(pen, _startPosX + 27, _startPosY + 45, _startPosX + 33, _startPosY + 45);

                g.DrawLine(pen, _startPosX + 50, _startPosY + 42, _startPosX + 50, _startPosY + 48);
                g.DrawLine(pen, _startPosX + 47, _startPosY + 45, _startPosX + 53, _startPosY + 45);
            }
            // отвал
            if (bulldoser.Blade)
            {
                g.FillRectangle(additionalBrush, _startPosX + 82, _startPosY + 30, 10, 40);
                g.FillRectangle(additionalBrush, _startPosX + 82, _startPosY + 60, 40, 10);

                for (int i = 0; i < 15; i++)
                {
                    g.FillRectangle(additionalBrush, _startPosX + 82, _startPosY + 60 - i * 2, 40 - i * 2, 10);
                }

                g.FillRectangle(additionalBrush, _startPosX + 50, _startPosY + 25, 10, 10);

                for (int i = 0; i < 16; i++)
                {
                    g.FillRectangle(additionalBrush, _startPosX + 50 + i * 2, _startPosY + 25 + i, 10, 10);
                }
            }
        }

    }
}
