
using ProjectTractor.MovementStrategy;
using ProjectTractor.DrawningObjects;

namespace ProjectTractor
{

    /// <summary>
    ///     " "
    /// </summary>
    public partial class FormTractor : Form
    {
        private PictureBox pictureBoxTractor;
        private Button buttonCreateTractor;
        private Button buttonTop;
        private Button buttonLeft;
        private Button buttonDown;
        private Button buttonCreateBulldoserTractor;
        private Button buttonStep;
        private ComboBox comboBoxStrategy;
        private Button buttonRight;

        /// <summary>
        /// -   
        /// </summary>
        private DrawningTractor? _drawningTractor;
        private Button ButtonSelectTractor;

        /// <summary>
        ///  
        /// </summary>
        private AbstractStrategy? _strategy;
        /// <summary>
        ///  
        /// </summary>
        public DrawningTractor? SelectedTractor { get; private set; }
        /// <summary>
        ///  
        /// </summary>
        public FormTractor()
        {
            InitializeComponent();
            _strategy = null;
            SelectedTractor = null;
        }
        /// <summary>
        ///   
        /// </summary>
        private void Draw()
        {
            if (_drawningTractor == null)
            {
                return;
            }
            Bitmap bmp = new(pictureBoxTractor.Width,
            pictureBoxTractor.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _drawningTractor.DrawTransport(gr);
            pictureBoxTractor.Image = bmp;
        }
        /// <summary>
        ///   
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonMove_Click(object sender, EventArgs e)
        {
            if (_drawningTractor == null)
            {
                return;
            }
            string name = ((Button)sender)?.Name ?? string.Empty;
            switch (name)
            {
                case "buttonUp":
                    _drawningTractor.MoveTransport(DirectionType.Up);
                    break;
                case "buttonDown":
                    _drawningTractor.MoveTransport(DirectionType.Down);
                    break;
                case "buttonLeft":
                    _drawningTractor.MoveTransport(DirectionType.Left);
                    break;
                case "buttonRight":
                    _drawningTractor.MoveTransport(DirectionType.Right);
                    break;
            }
            Draw();
        }
        /// <summary>
        ///   
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonCreate_Click(object sender, EventArgs e)
        {
            Random random = new();
            Color color = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
            ColorDialog dialog = new();
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                color = dialog.Color;
            }
            _drawningTractor = new DrawningTractor(random.Next(100, 300),
            random.Next(1000, 3000), color,
            pictureBoxTractor.Width, pictureBoxTractor.Height);
            _drawningTractor.SetPosition(random.Next(10, 100), random.Next(10,
            100));
            Draw();
        }
        /// <summary>
        ///   
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonStartegyStep_Click(object sender, EventArgs e)
        {
            if (_drawningTractor == null)
            {
                return;
            }
            if (comboBoxStrategy.Enabled)
            {
                _strategy = comboBoxStrategy.SelectedIndex switch
                {
                    0 => new MoveToCenter(),
                    1 => new MoveToBorder(),
                    _ => null,
                };
                if (_strategy == null)
                {
                    return;
                }
                _strategy.SetData(_drawningTractor.GetMoveableObject,
                pictureBoxTractor.Width, pictureBoxTractor.Height);
            }
            if (_strategy == null)
            {
                return;
            }
            comboBoxStrategy.Enabled = false;
            _strategy.MakeStep();
            Draw();
            if (_strategy.GetStatus() == Status.Finish)
            {
                comboBoxStrategy.Enabled = true;
                _strategy = null;
            }
        }
        /// <summary>
        ///  
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonSelectTractor_Click(object sender, EventArgs e)
        {
            SelectedTractor = _drawningTractor;
            DialogResult = DialogResult.OK;
        }

        private void InitializeComponent()
        {
            this.pictureBoxTractor = new System.Windows.Forms.PictureBox();
            this.buttonCreateTractor = new System.Windows.Forms.Button();
            this.buttonTop = new System.Windows.Forms.Button();
            this.buttonLeft = new System.Windows.Forms.Button();
            this.buttonDown = new System.Windows.Forms.Button();
            this.buttonRight = new System.Windows.Forms.Button();
            this.buttonCreateBulldoserTractor = new System.Windows.Forms.Button();
            this.buttonStep = new System.Windows.Forms.Button();
            this.comboBoxStrategy = new System.Windows.Forms.ComboBox();
            this.ButtonSelectTractor = new System.Windows.Forms.Button();
            ((System.ComponentModel.ISupportInitialize)(this.pictureBoxTractor)).BeginInit();
            this.SuspendLayout();
            // 
            // pictureBoxTractor
            // 
            this.pictureBoxTractor.Dock = System.Windows.Forms.DockStyle.Fill;
            this.pictureBoxTractor.Location = new System.Drawing.Point(0, 0);
            this.pictureBoxTractor.Name = "pictureBoxTractor";
            this.pictureBoxTractor.Size = new System.Drawing.Size(882, 453);
            this.pictureBoxTractor.SizeMode = System.Windows.Forms.PictureBoxSizeMode.AutoSize;
            this.pictureBoxTractor.TabIndex = 0;
            this.pictureBoxTractor.TabStop = false;
            // 
            // buttonCreateTractor
            // 
            this.buttonCreateTractor.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left)));
            this.buttonCreateTractor.Location = new System.Drawing.Point(12, 403);
            this.buttonCreateTractor.Name = "buttonCreateTractor";
            this.buttonCreateTractor.Size = new System.Drawing.Size(246, 29);
            this.buttonCreateTractor.TabIndex = 1;
            this.buttonCreateTractor.Text = " ";
            this.buttonCreateTractor.UseVisualStyleBackColor = true;
            this.buttonCreateTractor.Click += new System.EventHandler(this.ButtonCreate_Click);
            // 
            // buttonTop
            // 
            this.buttonTop.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
            this.buttonTop.BackgroundImage = global::ProjectTractor.Properties.Resources.arrowUp;
            this.buttonTop.BackgroundImageLayout = System.Windows.Forms.ImageLayout.Stretch;
            this.buttonTop.Location = new System.Drawing.Point(804, 375);
            this.buttonTop.Name = "buttonTop";
            this.buttonTop.Size = new System.Drawing.Size(30, 30);
            this.buttonTop.TabIndex = 2;
            this.buttonTop.UseVisualStyleBackColor = true;
            this.buttonTop.Click += new System.EventHandler(this.ButtonMove_Click);
            // 
            // buttonLeft
            // 
            this.buttonLeft.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
            this.buttonLeft.BackgroundImage = global::ProjectTractor.Properties.Resources.arrowLeft;
            this.buttonLeft.BackgroundImageLayout = System.Windows.Forms.ImageLayout.Stretch;
            this.buttonLeft.Location = new System.Drawing.Point(768, 411);
            this.buttonLeft.Name = "buttonLeft";
            this.buttonLeft.Size = new System.Drawing.Size(30, 30);
            this.buttonLeft.TabIndex = 3;
            this.buttonLeft.UseVisualStyleBackColor = true;
            this.buttonLeft.Click += new System.EventHandler(this.ButtonMove_Click);
            // 
            // buttonDown
            // 
            this.buttonDown.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
            this.buttonDown.BackgroundImage = global::ProjectTractor.Properties.Resources.arrowDown;
            this.buttonDown.BackgroundImageLayout = System.Windows.Forms.ImageLayout.Stretch;
            this.buttonDown.Location = new System.Drawing.Point(804, 411);
            this.buttonDown.Name = "buttonDown";
            this.buttonDown.Size = new System.Drawing.Size(30, 30);
            this.buttonDown.TabIndex = 4;
            this.buttonDown.UseVisualStyleBackColor = true;
            this.buttonDown.Click += new System.EventHandler(this.ButtonMove_Click);
            // 
            // buttonRight
            // 
            this.buttonRight.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
            this.buttonRight.BackgroundImage = global::ProjectTractor.Properties.Resources.arrowRight;
            this.buttonRight.BackgroundImageLayout = System.Windows.Forms.ImageLayout.Stretch;
            this.buttonRight.Location = new System.Drawing.Point(840, 411);
            this.buttonRight.Name = "buttonRight";
            this.buttonRight.Size = new System.Drawing.Size(30, 30);
            this.buttonRight.TabIndex = 5;
            this.buttonRight.UseVisualStyleBackColor = true;
            this.buttonRight.Click += new System.EventHandler(this.ButtonMove_Click);
            // 
            // buttonCreateBulldoserTractor
            // 
            this.buttonCreateBulldoserTractor.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left)));
            this.buttonCreateBulldoserTractor.Location = new System.Drawing.Point(264, 403);
            this.buttonCreateBulldoserTractor.Name = "buttonCreateBulldoserTractor";
            this.buttonCreateBulldoserTractor.Size = new System.Drawing.Size(199, 29);
            this.buttonCreateBulldoserTractor.TabIndex = 6;
            this.buttonCreateBulldoserTractor.Text = " ";
            this.buttonCreateBulldoserTractor.UseVisualStyleBackColor = true;
            this.buttonCreateBulldoserTractor.Click += new System.EventHandler(this.buttonCreateBulldoserTractor_Click);
            // 
            // buttonStep
            // 
            this.buttonStep.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.buttonStep.Location = new System.Drawing.Point(776, 46);
            this.buttonStep.Name = "buttonStep";
            this.buttonStep.Size = new System.Drawing.Size(94, 29);
            this.buttonStep.TabIndex = 7;
            this.buttonStep.Text = "";
            this.buttonStep.UseVisualStyleBackColor = true;
            this.buttonStep.Click += new System.EventHandler(this.buttonStep_Click);
            // 
            // comboBoxStrategy
            // 
            this.comboBoxStrategy.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.comboBoxStrategy.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.comboBoxStrategy.FormattingEnabled = true;
            this.comboBoxStrategy.Items.AddRange(new object[] {
            " ",
            " "});
            this.comboBoxStrategy.Location = new System.Drawing.Point(719, 12);
            this.comboBoxStrategy.Name = "comboBoxStrategy";
            this.comboBoxStrategy.Size = new System.Drawing.Size(151, 28);
            this.comboBoxStrategy.TabIndex = 8;
            // 
            // ButtonSelectTractor
            // 
            this.ButtonSelectTractor.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left)));
            this.ButtonSelectTractor.Location = new System.Drawing.Point(469, 403);
            this.ButtonSelectTractor.Name = "ButtonSelectTractor";
            this.ButtonSelectTractor.Size = new System.Drawing.Size(199, 29);
            this.ButtonSelectTractor.TabIndex = 9;
            this.ButtonSelectTractor.Text = " ";
            this.ButtonSelectTractor.UseVisualStyleBackColor = true;
            this.ButtonSelectTractor.Click += new System.EventHandler(this.ButtonSelectTractor_Click);
            // 
            // FormTractor
            // 
            this.ClientSize = new System.Drawing.Size(882, 453);
            this.Controls.Add(this.ButtonSelectTractor);
            this.Controls.Add(this.comboBoxStrategy);
            this.Controls.Add(this.buttonStep);
            this.Controls.Add(this.buttonCreateBulldoserTractor);
            this.Controls.Add(this.buttonRight);
            this.Controls.Add(this.buttonDown);
            this.Controls.Add(this.buttonLeft);
            this.Controls.Add(this.buttonTop);
            this.Controls.Add(this.buttonCreateTractor);
            this.Controls.Add(this.pictureBoxTractor);
            this.Name = "FormTractor";
            ((System.ComponentModel.ISupportInitialize)(this.pictureBoxTractor)).EndInit();
            this.ResumeLayout(false);
            this.PerformLayout();

        }

        private void buttonCreateBulldoserTractor_Click(object sender, EventArgs e)
        {
            Random random = new();
            Color color = Color.FromArgb(random.Next(0, 256),
            random.Next(0, 256), random.Next(0, 256));
            ColorDialog dialogColor = new();
            if (dialogColor.ShowDialog() == DialogResult.OK)
            {
                color = dialogColor.Color;
            }
            Color dopColor = Color.FromArgb(random.Next(0, 256),
            random.Next(0, 256), random.Next(0, 256));
            ColorDialog dialogDopColor = new();
            if (dialogDopColor.ShowDialog() == DialogResult.OK)
            {
                dopColor = dialogDopColor.Color;
            }
            _drawningTractor = new DrawningBulldoser(random.Next(100, 300),
            random.Next(1000, 3000),
            color,
            dopColor,
            Convert.ToBoolean(random.Next(0, 2)),
            Convert.ToBoolean(random.Next(0, 2)),
            pictureBoxTractor.Width,
            pictureBoxTractor.Height);
            _drawningTractor.SetPosition(random.Next(10, 100), random.Next(10, 100));
            Draw();
        }

        private void buttonStep_Click(object sender, EventArgs e)
        {
            if (_drawningTractor == null)
            {
                return;
            }
            if (comboBoxStrategy.Enabled)
            {
                _strategy = comboBoxStrategy.SelectedIndex
                switch
                {
                    0 => new MoveToCenter(),
                    1 => new MoveToBorder(),
                    _ => null,
                };
                if (_strategy == null)
                {
                    return;
                }
                _strategy.SetData(new
                DrawningObjectTractor(_drawningTractor), pictureBoxTractor.Width,
                pictureBoxTractor.Height);
                comboBoxStrategy.Enabled = false;
            }
            if (_strategy == null)
            {
                return;
            }
            _strategy.MakeStep();
            Draw();
            if (_strategy.GetStatus() == Status.Finish)
            {
                comboBoxStrategy.Enabled = true;
                _strategy = null;
            }
        }
    }
}