﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using ProjectTractor.DrawningObjects;
using ProjectTractor.Generics;
using ProjectTractor.MovementStrategy;

namespace ProjectTractor
{
    public partial class FormTractorCollection : Form
    { 

        /// <summary>
        /// Набор объектов
        /// </summary>
        private readonly TractorsGenericCollection<DrawningTractor,
        DrawningObjectTractor> _tractors;
        /// <summary>
        /// Конструктор
        /// </summary>
        public FormTractorCollection()
        {
            InitializeComponent();
            _tractors = new TractorsGenericCollection<DrawningTractor,
            DrawningObjectTractor>(pictureBoxCollection.Width, pictureBoxCollection.Height);
        }

        private void ButtonAddTractor_Click(object sender, EventArgs e)
        {
            FormTractor form = new();
            if (form.ShowDialog() == DialogResult.OK)
            {
                if (_tractors + form.SelectedTractor)
                {
                    MessageBox.Show("Объект добавлен");
                    pictureBoxCollection.Image = _tractors.ShowTractors();
                }
                else
                {
                    MessageBox.Show("Не удалось добавить объект");
                }
            }
        }

        /// <summary>
        /// Удаление объекта из набора
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonRemoveTractor_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("Удалить объект?", "Удаление",
            MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }
            int pos = Convert.ToInt32(maskedTextBoxNumber.Text);
            if (_tractors - pos)
            {
                MessageBox.Show("Объект удален");
                pictureBoxCollection.Image = _tractors.ShowTractors();
            }
            else
            {
                MessageBox.Show("Не удалось удалить объект");
            }
        }
        /// <summary>
        /// Обновление рисунка по набору
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonRefreshCollection_Click(object sender, EventArgs
        e)
        {
            pictureBoxCollection.Image = _tractors.ShowTractors();
        }

        private void InitializeComponent()
        {
            this.pictureBoxCollection = new System.Windows.Forms.PictureBox();
            this.groupBox = new System.Windows.Forms.GroupBox();
            this.ButtonRefreshCollection = new System.Windows.Forms.Button();
            this.ButtonRemoveTractor = new System.Windows.Forms.Button();
            this.ButtonAddTractor = new System.Windows.Forms.Button();
            this.maskedTextBoxNumber = new System.Windows.Forms.MaskedTextBox();
            ((System.ComponentModel.ISupportInitialize)(this.pictureBoxCollection)).BeginInit();
            this.groupBox.SuspendLayout();
            this.SuspendLayout();
            // 
            // pictureBoxCollection
            // 
            this.pictureBoxCollection.Dock = System.Windows.Forms.DockStyle.Left;
            this.pictureBoxCollection.Location = new System.Drawing.Point(0, 0);
            this.pictureBoxCollection.Name = "pictureBoxCollection";
            this.pictureBoxCollection.Size = new System.Drawing.Size(620, 437);
            this.pictureBoxCollection.TabIndex = 1;
            this.pictureBoxCollection.TabStop = false;
            // 
            // groupBox
            // 
            this.groupBox.Controls.Add(this.ButtonRefreshCollection);
            this.groupBox.Controls.Add(this.ButtonRemoveTractor);
            this.groupBox.Controls.Add(this.ButtonAddTractor);
            this.groupBox.Controls.Add(this.maskedTextBoxNumber);
            this.groupBox.Location = new System.Drawing.Point(626, 12);
            this.groupBox.Name = "groupBox";
            this.groupBox.Size = new System.Drawing.Size(255, 413);
            this.groupBox.TabIndex = 2;
            this.groupBox.TabStop = false;
            this.groupBox.Text = "Инструменты";
            // 
            // ButtonRefreshCollection
            // 
            this.ButtonRefreshCollection.Location = new System.Drawing.Point(20, 331);
            this.ButtonRefreshCollection.Name = "ButtonRefreshCollection";
            this.ButtonRefreshCollection.Size = new System.Drawing.Size(211, 35);
            this.ButtonRefreshCollection.TabIndex = 8;
            this.ButtonRefreshCollection.Text = "Обновить коллекцию";
            this.ButtonRefreshCollection.UseVisualStyleBackColor = true;
            this.ButtonRefreshCollection.Click += new System.EventHandler(this.ButtonRefreshCollection_Click);
            // 
            // ButtonRemoveTractor
            // 
            this.ButtonRemoveTractor.Location = new System.Drawing.Point(28, 156);
            this.ButtonRemoveTractor.Name = "ButtonRemoveTractor";
            this.ButtonRemoveTractor.Size = new System.Drawing.Size(203, 33);
            this.ButtonRemoveTractor.TabIndex = 7;
            this.ButtonRemoveTractor.Text = "Удалить трактор";
            this.ButtonRemoveTractor.UseVisualStyleBackColor = true;
            this.ButtonRemoveTractor.Click += new System.EventHandler(this.ButtonRemoveTractor_Click);
            // 
            // ButtonAddTractor
            // 
            this.ButtonAddTractor.Location = new System.Drawing.Point(28, 46);
            this.ButtonAddTractor.Name = "ButtonAddTractor";
            this.ButtonAddTractor.Size = new System.Drawing.Size(203, 34);
            this.ButtonAddTractor.TabIndex = 6;
            this.ButtonAddTractor.Text = "Добавить трактор";
            this.ButtonAddTractor.UseVisualStyleBackColor = true;
            this.ButtonAddTractor.Click += new System.EventHandler(this.ButtonAddTractor_Click);
            // 
            // maskedTextBoxNumber
            // 
            this.maskedTextBoxNumber.Location = new System.Drawing.Point(28, 113);
            this.maskedTextBoxNumber.Name = "maskedTextBoxNumber";
            this.maskedTextBoxNumber.Size = new System.Drawing.Size(203, 27);
            this.maskedTextBoxNumber.TabIndex = 5;
            // 
            // FormTractorCollection
            // 
            this.ClientSize = new System.Drawing.Size(893, 437);
            this.Controls.Add(this.groupBox);
            this.Controls.Add(this.pictureBoxCollection);
            this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedSingle;
            this.MinimizeBox = false;
            this.Name = "FormTractorCollection";
            this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
            this.Text = "Набор тракторов";
            ((System.ComponentModel.ISupportInitialize)(this.pictureBoxCollection)).EndInit();
            this.groupBox.ResumeLayout(false);
            this.groupBox.PerformLayout();
            this.ResumeLayout(false);

        }

        private GroupBox groupBox;
        private Button ButtonRefreshCollection;
        private Button ButtonRemoveTractor;
        private Button ButtonAddTractor;
        private MaskedTextBox maskedTextBoxNumber;
        private PictureBox pictureBoxCollection;

    }
}
