﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectTractor.Generics
{
    internal class SetGeneric<T>
        where T : class
    {
        /// <summary>
        /// Массив объектов, которые храним
        /// </summary>
        private readonly T?[] _places;
        /// <summary>
        /// Количество объектов в массиве
        /// </summary>
        public int Count => _places.Length;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="count"></param>
        public SetGeneric(int count)
        {
            _places = new T?[count];
        }
        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="tractor">Добавляемый трактор</param>
        /// <returns></returns>
        public bool Insert(T tractor)
        {
            if (_places[0] == null)
            {
                _places[0] = tractor;
            }
            else
            {
                return Insert(tractor, 0);
            }
            return true;
        }
        /// <summary>
        /// Добавление объекта в набор на конкретную позицию
        /// </summary>
        /// <param name="tractor">Добавляемый трактор</param>
        /// <param name="position">Позиция</param>
        /// <returns></returns>
        public bool Insert(T tractor, int position)
        {

            if (!(position >= 0 && position < Count))
                return false;
            if (_places[position] != null)
            {
                int ind = position;
                while (ind < Count && _places[ind] != null)
                    ind++;
                if (ind == Count)
                    return false;
                for (int i = ind - 1; i >= position; i--)
                    _places[i + 1] = _places[i];
            }
            _places[position] = tractor;
            return true;
        }
        /// <summary>
        /// Удаление объекта из набора с конкретной позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public bool Remove(int position)
        {
            if (!(position >= 0 && position < Count))
            {
                return false;
            }
            _places[position] = null;
            return true;
        }
        /// <summary>
        /// Получение объекта из набора по позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T? Get(int position)
        {
            if (!(position >= 0 && position < Count))
            {
                return null;
            }
            return _places[position];
        }
    }
}
